' This class is a ComboBoxEx control, that implements a 
' control that is similar to the standard DropDownList control, with a 
' the following enhancements: you can assign all elements in one operation 
' as a comma-separated list to the ItemList property

' The real purpose of this sample is to show how you can implement
' features that most standard controls have, such as support for
' the AutoPostBack property and exposing of a postback event.

Imports System.ComponentModel
Imports System.Web.UI

<ToolboxData("<{0}:ComboBoxEx runat=server></{0}:ComboBoxEx>")> Public Class ComboBoxEx
    Inherits System.Web.UI.WebControls.WebControl
    Implements IPostBackDataHandler
    Implements IPostBackEventHandler

    ' These event definitions are equivalent.
    'Event SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs)
    Event SelectedIndexChanged As EventHandler

    ' The ItemList property
    ' Format is "option,value&option,value&...."
    Dim m_ItemList As String = "First,Second,Third"

    Property ItemList() As String
        Get
            Return m_ItemList
        End Get
        Set(ByVal Value As String)
            m_ItemList = Value
        End Set
    End Property

    ' The SelectedIndex property
    <DefaultValue(0)> _
    Property SelectedIndex() As Integer
        Get
            ' Notice how you can avoid to query ViewState twice
            Dim o As Object = Me.ViewState("SelectedIndex")
            If o Is Nothing Then
                Return 0   ' the default value
            Else
                Return CInt(o)
            End If
        End Get
        Set(ByVal Value As Integer)
            If Value >= 0 Then
                ' assign a new value only if it's a valid value
                Me.ViewState("SelectedIndex") = Value
            End If
        End Set
    End Property

    ' The AutoPostBack property

    Property AutoPostBack() As Boolean
        Get
            Dim o As Object = Me.ViewState("AutoPostBack")
            If o Is Nothing Then
                Return False
            Else
                Return CBool(o)
            End If
        End Get
        Set(ByVal Value As Boolean)
            Me.ViewState("AutoPostBack") = Value
        End Set
    End Property

    ' this is the method that overrides the default behavior and
    ' outputs the actual HTML code

    Protected Overrides Sub Render(ByVal output As System.Web.UI.HtmlTextWriter)
        ' get the array from the ItemList property
        Dim items() As String = ItemList.Split(","c)

        ' output the <select> tag and its attributes.
        output.AddAttribute("name", Me.UniqueID)
        Me.AddAttributesToRender(output)

        ' if this control requires auto postback
        If AutoPostBack Then
            ' add the two attributes that implement it.
            ' Note the GetPostBackEventReference method will actually
            ' output the javascript code for the __doPostBack routine.
            output.AddAttribute("onchange", Page.GetPostBackEventReference(Me))
            output.AddAttribute("language", "javascript")
        End If

        ' next methods outputs the SELECT tag and all the attributes
        ' defined previously
        output.RenderBeginTag("select")

        ' now we can add the individual <option> tags
        Dim index As Integer

        For index = 0 To items.Length - 1
            ' Add the Value attribute.
            output.AddAttribute("value", index.ToString)

            ' If this is the selected element, add a "selected" attributes
            If index > 0 AndAlso index = SelectedIndex Then
                output.AddAttribute("selected", "selected")
            End If

            ' Output the <option> tag (and all the attributes set previously)
            output.RenderBeginTag("option")
            output.Write(items(index))
            output.RenderEndTag()
        Next
        ' close the </select> tag.
        output.RenderEndTag()
    End Sub

    ' this method is invoked when a postback on the parent form occurs

    Public Function LoadPostData(ByVal postDataKey As String, ByVal postCollection As System.Collections.Specialized.NameValueCollection) As Boolean Implements System.Web.UI.IPostBackDataHandler.LoadPostData
        ' The post value is equal to the index of the selected element.
        Dim newSelectedIndex As Integer = CInt(postCollection(postDataKey))
        If SelectedIndex <> newSelectedIndex Then
            ' if the new value is different, store in the property.
            SelectedIndex = newSelectedIndex
            ' Tell ASP.NET to call the RaisePostDataChangedEvent method.
            Return True
        End If
    End Function

    ' this method is invoked when all the other controls have processed
    ' their LoadPostData method, and only if the LoadPostData method
    ' for this ComboBoxEx control returned True.

    Public Sub RaisePostDataChangedEvent() Implements System.Web.UI.IPostBackDataHandler.RaisePostDataChangedEvent
        ' raise an event in the parent form
        OnSelectedIndexChanged(EventArgs.Empty)
    End Sub

    ' inherited controls can override this method to control if and when
    ' they raise the SelectedIndexChanged event in their clients

    Protected Overridable Sub OnSelectedIndexChanged(ByVal e As EventArgs)
        RaiseEvent SelectedIndexChanged(Me, e)
    End Sub

    ' another public event
    Public Event Click As EventHandler

    ' this method is invoked if this control caused a postback
    Public Sub RaisePostBackEvent(ByVal eventArgument As String) Implements System.Web.UI.IPostBackEventHandler.RaisePostBackEvent
        OnClick(EventArgs.Empty)
    End Sub

    ' inherited controls can override this method to control if and when
    ' they raise the Click event in their clients

    Protected Overridable Sub OnClick(ByVal e As EventArgs)
        RaiseEvent Click(Me, e)
    End Sub

End Class
